@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2005, 2007, 2009, 2011, 2012  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007, 2009, 2011, 2012, 2014, 2015  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -38,7 +38,7 @@ fromtext_rrsig(ARGS_FROMTEXT) {
 	isc_buffer_t buffer;
 	isc_uint32_t time_signed, time_expire;
 
-	REQUIRE(type == 46);
+	REQUIRE(type == dns_rdatatype_rrsig);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -90,7 +90,20 @@ fromtext_rrsig(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	RETTOK(dns_time32_fromtext(DNS_AS_STR(token), &time_expire));
+	if (strlen(DNS_AS_STR(token)) <= 10U &&
+	    *DNS_AS_STR(token) != '-' && *DNS_AS_STR(token) != '+') {
+		char *end;
+		unsigned long u;
+		isc_uint64_t u64;
+
+		u64 = u = strtoul(DNS_AS_STR(token), &end, 10);
+		if (u == ULONG_MAX || *end != 0)
+			RETTOK(DNS_R_SYNTAX);
+		if (u64 > 0xffffffffUL)
+			RETTOK(ISC_R_RANGE);
+		time_expire = u;
+	} else
+		RETTOK(dns_time32_fromtext(DNS_AS_STR(token), &time_expire));
 	RETERR(uint32_tobuffer(time_expire, target));
 
 	/*
@@ -98,7 +111,20 @@ fromtext_rrsig(ARGS_FROMTEXT) {
 	 */
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      ISC_FALSE));
-	RETTOK(dns_time32_fromtext(DNS_AS_STR(token), &time_signed));
+	if (strlen(DNS_AS_STR(token)) <= 10U &&
+	    *DNS_AS_STR(token) != '-' && *DNS_AS_STR(token) != '+') {
+		char *end;
+		unsigned long u;
+		isc_uint64_t u64;
+
+		u64 = u = strtoul(DNS_AS_STR(token), &end, 10);
+		if (u == ULONG_MAX || *end != 0)
+			RETTOK(DNS_R_SYNTAX);
+		if (u64 > 0xffffffffUL)
+			RETTOK(ISC_R_RANGE);
+		time_signed = u;
+	} else
+		RETTOK(dns_time32_fromtext(DNS_AS_STR(token), &time_signed));
 	RETERR(uint32_tobuffer(time_signed, target));
 
 	/*
@@ -115,7 +141,8 @@ fromtext_rrsig(ARGS_FROMTEXT) {
 				      ISC_FALSE));
 	dns_name_init(&name, NULL);
 	buffer_fromregion(&buffer, &token.value.as_region);
-	origin = (origin != NULL) ? origin : dns_rootname;
+	if (origin == NULL)
+		origin = dns_rootname;
 	RETTOK(dns_name_fromtext(&name, &buffer, origin, options, target));
 
 	/*
@@ -135,7 +162,7 @@ totext_rrsig(ARGS_TOTEXT) {
 	unsigned long foot;
 	dns_name_t name;
 
-	REQUIRE(rdata->type == 46);
+	REQUIRE(rdata->type == dns_rdatatype_rrsig);
 	REQUIRE(rdata->length != 0);
 
 	dns_rdata_toregion(rdata, &sr);
@@ -152,7 +179,6 @@ totext_rrsig(ARGS_TOTEXT) {
 	if (dns_rdatatype_isknown(covered) && covered != 0) {
 		RETERR(dns_rdatatype_totext(covered, target));
 	} else {
-		char buf[sizeof("TYPE65535")];
 		sprintf(buf, "TYPE%u", covered);
 		RETERR(str_totext(buf, target));
 	}
@@ -239,7 +265,7 @@ fromwire_rrsig(ARGS_FROMWIRE) {
 	isc_region_t sr;
 	dns_name_t name;
 
-	REQUIRE(type == 46);
+	REQUIRE(type == dns_rdatatype_rrsig);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -282,7 +308,7 @@ towire_rrsig(ARGS_TOWIRE) {
 	dns_name_t name;
 	dns_offsets_t offsets;
 
-	REQUIRE(rdata->type == 46);
+	REQUIRE(rdata->type == dns_rdatatype_rrsig);
 	REQUIRE(rdata->length != 0);
 
 	dns_compress_setmethods(cctx, DNS_COMPRESS_NONE);
@@ -320,7 +346,7 @@ compare_rrsig(ARGS_COMPARE) {
 
 	REQUIRE(rdata1->type == rdata2->type);
 	REQUIRE(rdata1->rdclass == rdata2->rdclass);
-	REQUIRE(rdata1->type == 46);
+	REQUIRE(rdata1->type == dns_rdatatype_rrsig);
 	REQUIRE(rdata1->length != 0);
 	REQUIRE(rdata2->length != 0);
 
@@ -333,7 +359,7 @@ static inline isc_result_t
 fromstruct_rrsig(ARGS_FROMSTRUCT) {
 	dns_rdata_rrsig_t *sig = source;
 
-	REQUIRE(type == 46);
+	REQUIRE(type == dns_rdatatype_rrsig);
 	REQUIRE(source != NULL);
 	REQUIRE(sig->common.rdtype == type);
 	REQUIRE(sig->common.rdclass == rdclass);
@@ -394,7 +420,7 @@ tostruct_rrsig(ARGS_TOSTRUCT) {
 	dns_rdata_rrsig_t *sig = target;
 	dns_name_t signer;
 
-	REQUIRE(rdata->type == 46);
+	REQUIRE(rdata->type == dns_rdatatype_rrsig);
 	REQUIRE(target != NULL);
 	REQUIRE(rdata->length != 0);
 
@@ -475,7 +501,7 @@ freestruct_rrsig(ARGS_FREESTRUCT) {
 	dns_rdata_rrsig_t *sig = (dns_rdata_rrsig_t *) source;
 
 	REQUIRE(source != NULL);
-	REQUIRE(sig->common.rdtype == 46);
+	REQUIRE(sig->common.rdtype == dns_rdatatype_rrsig);
 
 	if (sig->mctx == NULL)
 		return;
@@ -488,7 +514,7 @@ freestruct_rrsig(ARGS_FREESTRUCT) {
 
 static inline isc_result_t
 additionaldata_rrsig(ARGS_ADDLDATA) {
-	REQUIRE(rdata->type == 46);
+	REQUIRE(rdata->type == dns_rdatatype_rrsig);
 
 	UNUSED(rdata);
 	UNUSED(add);
@@ -500,7 +526,7 @@ additionaldata_rrsig(ARGS_ADDLDATA) {
 static inline isc_result_t
 digest_rrsig(ARGS_DIGEST) {
 
-	REQUIRE(rdata->type == 46);
+	REQUIRE(rdata->type == dns_rdatatype_rrsig);
 
 	UNUSED(rdata);
 	UNUSED(digest);
@@ -514,7 +540,7 @@ covers_rrsig(dns_rdata_t *rdata) {
 	dns_rdatatype_t type;
 	isc_region_t r;
 
-	REQUIRE(rdata->type == 46);
+	REQUIRE(rdata->type == dns_rdatatype_rrsig);
 
 	dns_rdata_toregion(rdata, &r);
 	type = uint16_fromregion(&r);
@@ -525,7 +551,7 @@ covers_rrsig(dns_rdata_t *rdata) {
 static inline isc_boolean_t
 checkowner_rrsig(ARGS_CHECKOWNER) {
 
-	REQUIRE(type == 46);
+	REQUIRE(type == dns_rdatatype_rrsig);
 
 	UNUSED(name);
 	UNUSED(type);
@@ -538,7 +564,7 @@ checkowner_rrsig(ARGS_CHECKOWNER) {
 static inline isc_boolean_t
 checknames_rrsig(ARGS_CHECKNAMES) {
 
-	REQUIRE(rdata->type == 46);
+	REQUIRE(rdata->type == dns_rdatatype_rrsig);
 
 	UNUSED(rdata);
 	UNUSED(owner);
@@ -557,7 +583,7 @@ casecompare_rrsig(ARGS_COMPARE) {
 
 	REQUIRE(rdata1->type == rdata2->type);
 	REQUIRE(rdata1->rdclass == rdata2->rdclass);
-	REQUIRE(rdata1->type == 46);
+	REQUIRE(rdata1->type == dns_rdatatype_rrsig);
 	REQUIRE(rdata1->length != 0);
 	REQUIRE(rdata2->length != 0);
 
