@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2005, 2007-2009, 2011, 2012  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007-2009, 2011, 2012, 2014, 2015, 2017  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1998-2002  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -28,6 +28,7 @@
 #include <isc/mem.h>
 #include <isc/msgs.h>
 #include <isc/platform.h>
+#include <isc/print.h>
 #include <isc/task.h>
 #include <isc/thread.h>
 #include <isc/time.h>
@@ -130,12 +131,12 @@ struct isc__timermgr {
 
 ISC_TIMERFUNC_SCOPE isc_result_t
 isc__timer_create(isc_timermgr_t *manager, isc_timertype_t type,
-		  isc_time_t *expires, isc_interval_t *interval,
-		  isc_task_t *task, isc_taskaction_t action, const void *arg,
+		  const isc_time_t *expires, const isc_interval_t *interval,
+		  isc_task_t *task, isc_taskaction_t action, void *arg,
 		  isc_timer_t **timerp);
 ISC_TIMERFUNC_SCOPE isc_result_t
 isc__timer_reset(isc_timer_t *timer, isc_timertype_t type,
-		 isc_time_t *expires, isc_interval_t *interval,
+		 const  isc_time_t *expires, const isc_interval_t *interval,
 		 isc_boolean_t purge);
 ISC_TIMERFUNC_SCOPE isc_timertype_t
 isc__timer_gettype(isc_timer_t *timer);
@@ -392,8 +393,8 @@ destroy(isc__timer_t *timer) {
 
 ISC_TIMERFUNC_SCOPE isc_result_t
 isc__timer_create(isc_timermgr_t *manager0, isc_timertype_t type,
-		  isc_time_t *expires, isc_interval_t *interval,
-		  isc_task_t *task, isc_taskaction_t action, const void *arg,
+		  const isc_time_t *expires, const isc_interval_t *interval,
+		  isc_task_t *task, isc_taskaction_t action, void *arg,
 		  isc_timer_t **timerp)
 {
 	isc__timermgr_t *manager = (isc__timermgr_t *)manager0;
@@ -514,7 +515,7 @@ isc__timer_create(isc_timermgr_t *manager0, isc_timertype_t type,
 
 ISC_TIMERFUNC_SCOPE isc_result_t
 isc__timer_reset(isc_timer_t *timer0, isc_timertype_t type,
-		 isc_time_t *expires, isc_interval_t *interval,
+		 const isc_time_t *expires, const isc_interval_t *interval,
 		 isc_boolean_t purge)
 {
 	isc__timer_t *timer = (isc__timer_t *)timer0;
@@ -692,7 +693,7 @@ dispatch(isc__timermgr_t *manager, isc_time_t *now) {
 
 	while (manager->nscheduled > 0 && !done) {
 		timer = isc_heap_element(manager->heap, 1);
-		INSIST(timer->type != isc_timertype_inactive);
+		INSIST(timer != NULL && timer->type != isc_timertype_inactive);
 		if (isc_time_compare(now, &timer->due) >= 0) {
 			if (timer->type == isc_timertype_ticker) {
 				type = ISC_TIMEREVENT_TICK;
@@ -937,6 +938,7 @@ isc__timermgr_create(isc_mem_t *mctx, isc_timermgr_t **managerp) {
 						ISC_MSG_FAILED, "failed"));
 		return (ISC_R_UNEXPECTED);
 	}
+	isc_thread_setname(manager->thread, "isc-timer");
 #endif
 #ifdef USE_SHARED_MANAGER
 	manager->refs = 1;
@@ -977,12 +979,13 @@ isc__timermgr_destroy(isc_timermgr_t **managerp) {
 	LOCK(&manager->lock);
 
 #ifdef USE_SHARED_MANAGER
-	if (manager->refs > 1) {
-		manager->refs--;
+	manager->refs--;
+	if (manager->refs > 0) {
 		UNLOCK(&manager->lock);
 		*managerp = NULL;
 		return;
 	}
+	timermgr = NULL;
 #endif /* USE_SHARED_MANAGER */
 
 #ifndef USE_TIMER_THREAD
@@ -1065,7 +1068,7 @@ isc__timermgr_dispatch(isc_timermgr_t *manager0) {
 
 #ifdef USE_TIMERIMPREGISTER
 isc_result_t
-isc__timer_register() {
+isc__timer_register(void) {
 	return (isc_timer_register(isc__timermgr_create));
 }
 #endif
